@@ -8,12 +8,16 @@ complexity stamped out (well, eventually :).
 
 """
 
-__version__ = "0.8"
-__revision__ = "$Rev: 220 $"
-__date__ = "$Date: 2005-12-02 02:53:02 -0500 (Fri, 02 Dec 2005) $"
-__author__ = "Ryan Tomayko (rtomayko@gmail.com)"
-__copyright__ = "Copyright 2004-2005, Ryan Tomayko"
-__license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
+__revision__ = "$Rev: 273 $"
+__date__ = "$Date: 2006-02-26 20:27:28 +0000 (Sun, 26 Feb 2006) $"
+
+import release
+__version__ = release.version
+__author__ = release.author
+__email__ = release.email
+__copyright__ = release.copyright
+__license__ = release.license
+
 
 import sys
 import os
@@ -91,7 +95,7 @@ def load_template(file, name='', cache=1, encoding=None):
       byte-compiled version after compiling. This argument is ignored if
       file is not a filename.
     """
-    if isinstance(file, (str, unicode)):
+    if isinstance(file, basestring):
         if xml_sniff(file):
             fo = QuickTextReader(file)
             filename = '<string>'
@@ -154,7 +158,7 @@ def Template(file=None, source=None, name=None, **kw):
     elif file is not None:
         mod = load_template(file)
     elif source is not None:
-        mod = load_template(QuickTextReader(source))
+        mod = load_template(QuickTextReader(source), hex(id(source)))
     else:
         raise Exception("Must specify one of name, file, or source.")
     mod.Template.module = mod
@@ -310,7 +314,7 @@ class BaseTemplate(object):
         """
         if stream is None:
             stream = self.pull()
-        elif isinstance(stream, (str, unicode)):
+        elif isinstance(stream, basestring):
             if xml_sniff(stream):
                 stream = XML(stream, fragment=0)
             else:
@@ -342,7 +346,7 @@ class BaseTemplate(object):
     def _get_serializer(self, serializer):
         if serializer is None:
             return self.serializer
-        elif isinstance(serializer, (str, unicode)):
+        elif isinstance(serializer, basestring):
             return output_methods[serializer]
         else:
             return serializer
@@ -354,46 +358,36 @@ class BaseTemplate(object):
             return self.assume_encoding
         else:
             return assume_encoding
+    
+    def defined(self, name):
+        return hasattr(self, name)
+    
+    def value_of(self, name, default=None):
+        return getattr(self, name, default)
             
 class TemplatePath(object):
     def __init__(self, paths=None):
-        from os.path import normpath, expanduser, abspath
-        if isinstance(paths, (str, unicode)):
+        if isinstance(paths, basestring):
             paths = [paths]
         elif paths is None:
             paths = []
         paths += [os.getcwd(), '/']
-        self.paths = [abspath(normpath(expanduser(p))) for p in paths]
-        self.cache = {}
-    
-    def relativize(self, file, path):
-        from os.path import normpath, join, dirname, abspath, split, sep
-        head, tail = (dirname(abspath(file)), '')
-        parts = path.split(sep)
-        paths = self.paths
-        while 1:
-            if head in paths or head == '/':
-                return join(*parts)
-            head, tail = split(head)
-            parts.insert(0, tail)
-        
-    def insert(self, path, pos=0):
+        self.paths = [self._cleanse_path(p) for p in paths]
+
+    def _cleanse_path(self, path):
         from os.path import normpath, expanduser, abspath
-        self.paths.insert(pos, abspath(normpath(expanduser(path))))
-        self.cache = {}
+        return abspath(normpath(expanduser(path)))
+    
+    def insert(self, path, pos=0):
+        self.paths.insert(pos, self._cleanse_path(path))
     
     def append(self, path):
-        self.insert(path, len(self.paths))
+        self.paths.append(self._cleanse_path(path))
     
-    def find(self, path, rel=None):
-        cache = self.cache
-        from os.path import normpath, join, exists
+    def find(self, path, rel="/"):
+        from os.path import normpath, join, exists, dirname
         path = normpath(path)
-        if rel:
-            path = self.relativize(rel, path)
-        if path in cache:
-            return cache[path]
-        for p in self.paths:
+        for p in self.paths + [dirname(rel)]:
             p = join(p, path)
             if exists(p):
                 return p
