@@ -13,8 +13,8 @@ This module is based heavily on code from the Quixote ptl_import module by MEMS
 Exchange -- thanks guys.
 """
 
-__revision__ = "$Rev: 214 $"
-__date__ = "$Date: 2005-11-21 22:51:13 -0500 (Mon, 21 Nov 2005) $"
+__revision__ = "$Rev: 252 $"
+__date__ = "$Date: 2006-01-15 19:55:12 +0000 (Sun, 15 Jan 2006) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 
@@ -37,11 +37,10 @@ __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 # SOFTWARE.
 
 
+import os
 import sys
-import os.path
+import time
 import imp, ihooks, new
-import struct
-import marshal
 import __builtin__
 
 import kid.compiler
@@ -83,13 +82,12 @@ def get_template_name(name, filename):
     
 def _create_module(code, name, filename, store=1):
     name = get_template_name(name, filename)
-    mod = sys.modules.get(name, None)
-    if mod is None:
-        mod = new.module(name)
-        if store: sys.modules[name] = mod
-    mod.__name__ = name
+    mod = new.module(name)
     mod.__file__ = filename
+    mod.__ctime__ = time.time()
     exec code in mod.__dict__
+    if store:
+        sys.modules[name] = mod
     return mod
 
 # this is put in a pyc file to signal that it is a kid file
