@@ -30,7 +30,7 @@ class Namespace(object):
     def __repr__(self):
         return 'Namespace(%r, %r)' % (self.uri, self.prefix)
     def __equals__(self, other):
-        if isinstance(other, (str, unicode)):
+        if isinstance(other, basestring):
             return self.uri == other
         elif isinstance(other, Namespace):
             return self.uri == other.uri
