@@ -5,8 +5,8 @@ Parses Kid embedded XML to Python source code.
 
 from __future__ import generators
 
-__revision__ = "$Rev: 219 $"
-__date__ = "$Date: 2005-11-29 23:58:58 -0500 (Tue, 29 Nov 2005) $"
+__revision__ = "$Rev: 307 $"
+__date__ = "$Date: 2006-04-06 11:59:35 +0000 (Thu, 06 Apr 2006) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 __license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
@@ -30,6 +30,7 @@ QNAME_MATCH = kidns['match']
 QNAME_STRIP = kidns['strip']
 QNAME_ATTRIBUTES = kidns['attrs']
 QNAME_EXTENDS = kidns['extends']
+QNAME_LAYOUT = kidns['layout']
 
 # deprectaed
 QNAME_OMIT = kidns['omit']
@@ -53,7 +54,10 @@ def parse_file(filename, encoding=None):
 
     """
     source = open(filename, 'rb')
-    return parse(source, encoding, filename=filename)
+    try:
+        return parse(source, encoding, filename=filename)
+    finally:
+        source.close()
 
 class KidParser(object):
     def __init__(self, stream, encoding=None):
@@ -88,23 +92,25 @@ class KidParser(object):
         code.line('import kid')
         code.line('from kid.template_util import *')
         code.line('import kid.template_util as template_util')
+        code.line('_def_names = []')
 
         # default variables. can be overridden by template
         code.line('encoding = "%s"' % self.encoding)
         code.line('doctype = None')
         code.line('omit_namespaces = [kid.KID_XMLNS]')
+        code.line('layout_params = {}')
 
         # module methods.
         code.line('def pull(**kw): return Template(**kw).pull()')
-        code.line("def generate(encoding=encoding, **kw): "
-                  "return Template(**kw).generate(encoding=encoding)")
-        code.line("def serialize(encoding=encoding, **kw): "
-                  "return Template(**kw).serialize(encoding=encoding)")
-        code.line("def write(file, encoding=encoding, **kw): "
-                  "return Template(**kw).write(file, encoding=encoding)")
+        code.line("def generate(encoding=encoding, fragment=0, output=None, **kw): "
+                  "return Template(**kw).generate(encoding=encoding, fragment=fragment, output=output)")
+        code.line("def serialize(encoding=encoding, fragment=0, output=None, **kw): "
+                  "return Template(**kw).serialize(encoding=encoding, fragment=fragment, output=output)")
+        code.line("def write(file, encoding=encoding, fragment=0, output=None, **kw): "
+                  "return Template(**kw).write(file, encoding=encoding, fragment=fragment, output=output)")
         code.line('BaseTemplate = kid.BaseTemplate')
         code.line('def initialize(template): pass')
-        
+
         # expand code
         code = self.expand_code
         code.start_block('def initialize(self):')
@@ -113,7 +119,7 @@ class KidParser(object):
         code.end_block()
         code.start_block('def _pull(self):')
         # XXX hack: nasty kluge for making kwargs locals
-        code.line("exec template_util.get_locals(self)")
+        code.line("exec template_util.get_locals(self, locals())")
         code.line('current, ancestors = None, []')
         code.line('if doctype: yield (DOCTYPE, doctype)')
 
@@ -127,8 +133,19 @@ class KidParser(object):
         for (ev, item) in self.stream:
             if ev == START:
                 if item.tag == Comment:
+                    text = item.text.lstrip()
+                    if text.startswith('!'):
+                        continue
                     line = code.line
-                    line('_e = Comment(%r)' % item.text)
+                    if text.startswith('<') or text.startswith('['):
+                        sub = interpolate(item.text)
+                        if isinstance(sub, list):
+                            text = "''.join(%r)" % sub
+                        else:
+                            text = repr(sub)
+                    else:
+                        text = repr(item.text)
+                    line('_e = Comment(%s)' % text)
                     line('yield (START, _e); yield (END, _e); del _e')
                 elif item.tag == ProcessingInstruction:
                     if ' ' in item.text.strip():
@@ -145,7 +162,11 @@ class KidParser(object):
                         c.line('yield (START, _e); yield (END, _e); del _e')
                         del c
                 else:
+                    layout = None
                     if code is self.module_code:
+                        layout = item.get(QNAME_LAYOUT)
+                        if layout is not None:
+                            del item.attrib[QNAME_LAYOUT]
                         decl = ['class Template(']
                         extends = item.get(QNAME_EXTENDS)
                         parts = []
@@ -162,10 +183,28 @@ class KidParser(object):
                         code = self.expand_code
                         del decl, parts
                     self.def_proc(item, item.attrib, code)
+                    if layout is not None:
+                        old_code = code
+                        code = CodeGenerator(level=1)
+                        code.start_block("def _pull(self):")
+                        code.line('exec template_util.get_locals(self, locals())')
+                        code.line('kw = dict(layout_params)')
+                        code.line('kw.update(dict([(name, getattr(self, name)) for name in _def_names]))')
+                        code.line('kw.update(self.__dict__)')
+                        # XXX hack: this could be avoided if template args were not stored in self.__dict__
+                        # Note: these names cannot be passed to the layout template via layout_params
+                        code.line('kw.pop("assume_encoding", None)')
+                        code.line('kw.pop("_layout_classes", None)')
+                        code.line('temp = template_util.get_base_class(%s, __file__)(**kw)' % layout)
+                        code.line('temp._match_templates = self._match_templates + temp._match_templates')
+                        code.line('return temp._pull()')
+                        code.end_block()
+                        self.inst_defs.append(code)
+                        code = old_code
             elif ev == END and not item.tag in (ProcessingInstruction, Comment):
                 break
             elif ev == TEXT:
-                self.text_interpolate(ev, item, code)
+                self.text_interpolate(item, code)
             elif ev == XML_DECL and item[1] is not None:
                 self.module_code.line('encoding = %r' % item[1])
             elif ev == DOCTYPE:
@@ -193,12 +232,13 @@ class KidParser(object):
             code.start_block('def %s(*args, **kw):' % name)
             code.line('return Template().%s(*args, **kw)' % name)
             code.end_block()
+            code.line('_def_names.append("%s")' % name)
             self.module_defs.append(code)
 
             # instance method code
             code = CodeGenerator(level=1)
             code.start_block('def %s:' % class_decl)
-            code.line('exec template_util.get_locals(self)')
+            code.line('exec template_util.get_locals(self, locals())')
             code.line('current, ancestors = None, []')
             self.inst_defs.append(code)
             self.match_proc(item, attrib, code)
@@ -215,7 +255,7 @@ class KidParser(object):
             old_code = code
             code = CodeGenerator(level=1)
             code.start_block('def _match_func(self, item, apply):')
-            code.line('exec template_util.get_locals(self)')
+            code.line('exec template_util.get_locals(self, locals())')
             code.line('current, ancestors = None, []')
             self.for_proc(item, attrib, code)
             code.end_block()
@@ -306,7 +346,7 @@ class KidParser(object):
                 if isinstance(sub, list):
                     need_interpolation = 1
         expr = attrib.get(QNAME_ATTRIBUTES)
-        
+
         if expr is not None:
             del attrib[QNAME_ATTRIBUTES]
             attr_text = 'template_util.update_dict(%r, "%s", globals(), locals())' \
@@ -335,13 +375,12 @@ class KidParser(object):
             code.line('_cont = %s' % expr)
             return 1
 
-    def text_interpolate(self, ev, text, code):
+    def text_interpolate(self, text, code):
         interp = 0
         line = code.line
         sub = interpolate(text)
         if isinstance(sub, list):
-            code.start_block('for _e in %r:'\
-                             % sub)
+            code.start_block('for _e in %r:' % sub)
             code.line('for _e2 in template_util.generate_content(_e): yield _e2')
             code.end_block()
         else:
