@@ -11,7 +11,7 @@ from __future__ import generators
 from kid.et import *  # ElementTree
 from kid.util import open_resource, QuickTextReader
 
-import xml.parsers.expat
+from xml.parsers import expat
 
 # This is the default entity map.
 import htmlentitydefs
@@ -180,7 +180,10 @@ class ElementStream(object):
 def to_unicode(value, encoding):
     if isinstance(value, unicode):
         return value
-    
+
+    if hasattr(value, '__unicode__'):
+        return unicode(value)
+
     if not isinstance(value, str):
         value = str(value)
     
@@ -275,7 +278,7 @@ class ExpatParser(object):
             filename = '<string>'
         self._filename = filename
         self._source = source
-        self._parser = parser = xml.parsers.expat.ParserCreate(encoding, "}")
+        self._parser = parser = expat.ParserCreate(encoding, "}")
         self._queue = []
         
         # callbacks
@@ -308,7 +311,7 @@ class ExpatParser(object):
         self.external_dtd = default_external_dtd
         # setup entity handling
         self._parser.SetParamEntityParsing(
-            xml.parsers.expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
+            expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
         self._parser.ExternalEntityRefHandler = self._buildForeign
         self._parser.UseForeignDTD()
     
@@ -397,7 +400,7 @@ class ExpatParser(object):
     def feed(self, data):
         try:
             self._parser.Parse(data, 0)
-        except xml.parsers.expat.ExpatError, e:
+        except expat.ExpatError, e:
             e.filename = self._filename
             if hasattr(self, '_sourcetext'):
                 line = e.lineno
