@@ -52,11 +52,18 @@ class Serializer(object):
         return ''.join(text)
     
     def write(self, stream, file, encoding=None, fragment=0):
+        needs_closed = False
         if not hasattr(file, 'write'):
+            needs_closed = True
             file = open(file, 'wb')
-        write = file.write
-        for text in self.generate(stream, encoding, fragment):
-            write(text)
+        try:
+            write = file.write
+            for text in self.generate(stream, encoding, fragment):
+                write(text)
+        finally:
+            # only close a file if it was opened locally
+            if needs_closed:
+                file.close()
 
     def generate(self, stream, encoding=None, fragment=0):
         pass
@@ -91,7 +98,8 @@ class Serializer(object):
                 hops+=1
                 # pre-tag indentation was used.
                 # Make sure it's not used again for #PCDATA-sensitive elements
-                if ev == START and self.has_only_pcdata(item.tag):
+                if ev == START and (self.has_only_pcdata(item.tag)
+                        or item.tag == Comment):
                     text = ''
             else:
                 yield (ev, item)
@@ -116,7 +124,7 @@ class XMLSerializer(Serializer):
             self.decl = decl
         if doctype is not None:
             self.doctype = doctype
-        if isinstance(self.doctype, (str, unicode)):
+        if isinstance(self.doctype, basestring):
             # allow doctype strings
             self.doctype = doctypes[self.doctype]
         if namespaces:
@@ -265,16 +273,16 @@ class HTMLSerializer(Serializer):
         Serializer.__init__(self, encoding)
         if doctype:
             self.doctype = doctype
-        if isinstance(self.doctype, (str, unicode)):
+        if isinstance(self.doctype, basestring):
             # allow doctype strings
             self.doctype = doctypes[self.doctype]
         if transpose:
             self.transpose = transpose
 
     def has_only_pcdata(self, tagname):
-	if isinstance(tagname, (str,unicode)) and tagname[0] == '{':
+        if isinstance(tagname, basestring) and tagname[0] == '{':
             tagname = tagname.split('}')[1] 
-	    return tagname in self.elements_with_pcdata
+        return tagname in self.elements_with_pcdata
 
     def generate(self, stream, encoding=None, fragment=0):
         """Serializes an event stream to bytes of the specified encoding.
