@@ -15,7 +15,7 @@ output_dir = template_dir
 template_package = 'test.'
 
 test_modules = ['test_namespace', 'test_pull', 'test_compiler', 'test_parser',
-                'test_kid', 'test_serialization']
+                'test_kid', 'test_serialization', 'test_comment', 'test_layout']
 
 additional_tests = 0
 basic_tests = 0
