@@ -188,6 +188,15 @@ def diff_elm(elm1, elm2, diff_this=1):
     for elm1, elm2 in zip(ch1, ch2):
         diff_elm(elm1, elm2)
 
+def test_string_templates():
+    """Test for bug reported it ticket #70 where collisions occured in
+    templates created with a string.
+    """
+    t1 = kid.Template(source="<xx/>")
+    t2 = kid.Template(source="<yy/>")
+    assert str(t1) ==  '<?xml version="1.0" encoding="utf-8"?>\n<xx />'
+    assert str(t2) ==  '<?xml version="1.0" encoding="utf-8"?>\n<yy />'
+
 __tests__ = [test_import_and_expand,
              test_import_template_func,
              test_load_template_func,
